# -*- coding:Utf-8 -*-
#-------------------------------------------------------------------------------
# Nom :       LecteurDeConfig.py
# But :       Analysant un fichier à la manière des configurations de VALVE
#               - ignore les parties commentées
#               - ignore les espaces et les tabulations situées en début et en fin de ligne
#               - assure l'absence des retours chariots Windows et Linux
#
# Auteur :     Nicolous
#
# Créé le :    30/11/2007
# Copyright :  Aucun
# Licence :    Complétement libre
#-------------------------------------------------------------------------------

"""

Exemple d'utilisation du module :

    from LecteurDeConfig import LecteurDeConfig

    # Analyse un fichier à la manière de la console VALVE
    adminlist = LecteurDeConfig(es.getAddonPath('monaddon') + '/adminlist.txt')

    # Affiche la liste des lignes (ex: une liste de steamid) dans la console du serveur
    es.log(adminlist.getLignes())

    # Affiche dans la console du serveur le nom du fichier qui a été analysé
    es.log(adminlist.getNomFichier())

"""

import es
import re

class LecteurDeConfig():
    """
        Classe analysant un fichier à la manière des configigurations de VALVE

        @author Nicolous, reprenant l'un des concepts de DoCky
        @version 1
    """

    # Expression régulière d'un commentaire
    REGEX_COMMENTAIRES = re.compile("//.*")

    # Expression régulière des espaces et des tabulations situées en début et fin de ligne
    REGEX_ESPACES_TABULATIONS = re.compile("^(( |\t)+)|(( |\t)+)$")

    def __init__(self,nomFichier):
        """
            @param nomFichier Le nom du fichier

            Jette une exception de type IOError si la lecture du fichier pose problème
        """

        # Mémorisation du nom du fichier
        self.nomFichier = nomFichier

        try:
            fichier = open(nomFichier,"r")

            # Récupération des lignes du fichier sous forme de liste
            self.lignes = fichier.read().split("\n")

            # Lancement de l'analyse
            self.__analyse()

            fichier.close()
        except IOError:
            es.dbgmsg(0,"lecteurDeConfig.py : Impossible de lire le fichier " + nomFichier)
            raise

    def __analyse(self):
        """Procédure privée analysant les lignes récupérées"""

        index = 0
        while index < len(self.lignes):

            # Suppression de la partie commentée
            self.lignes[index] = self.__supprimeCommentaires(self.lignes[index])

            # Supression des espaces et des tabulations en début et fin de ligne
            self.lignes[index] = self.__aligneAGauche(self.lignes[index])

            # Suppression des retours chariots restants
            self.lignes[index] = self.__supprimeRetoursChariots(self.lignes[index])

            # Suppression des guillemets doubles
            self.lignes[index] = self.__supprimeGuillemetsDoubles(self.lignes[index])

            # Si il ne reste rien sur la ligne, on la retire de la liste des lignes
            if self.lignes[index] == "":
                self.lignes.remove(self.lignes[index])
            else:
                index += 1

    def __supprimeGuillemetsDoubles(self,ligne):
        """
            Fonction privée retournant une version sans guillemets doubles d'une ligne

             @param ligne Ligne à analyser
        """

        try:
            return ligne.replace('"','')
        except TypeError:
            es.dbgmsg(0,"lecteurDeConfig.py @ supprimeGuillemetsDoubles : le paramètre doit être de type str")
            raise

    def __supprimeCommentaires(self,ligne):
        """
            Fonction privée retournant une version non commentée d'une ligne

            @param ligne Ligne à analyser
        """

        try:
            return re.sub(self.__class__.REGEX_COMMENTAIRES,"",ligne)
        except TypeError:
            es.dbgmsg(0,"lecteurDeConfig.py @ supprimeCommentaires : le paramètre doit être de type str")
            raise

    def __supprimeRetoursChariots(self,ligne):
         """
            Fonction privée retournant une version sans retour chariot de type Windows et Linux

            @param ligne Ligne à analyser
         """

         try:
            return ligne.replace("\r","") # l'absence de "\n" est assurée dans le constructeur
         except TypeError:
            es.dbgmsg(0,"lecteurDeConfig.py @ supprimeRetoursChariots : le paramètre doit être de type str")
            raise

    def __aligneAGauche(self,ligne):
        """
            Fonction privée retournant une version sans espaces ni tabulations en début et fin de ligne

            @param ligne Ligne à analyser
        """

        try:
            return re.sub(self.__class__.REGEX_ESPACES_TABULATIONS,"",ligne)
        except TypeError:
            es.dbgmsg(0,"lecteurDeConfig.py @ aligneAGauche : le paramètre doit être de type str")
            raise

    def getNomFichier(self):
        """Fonction retournant le nom du fichier analysé"""

        return self.nomFichier

    def getLignes(self):
        """Fonction retournant la version analysées des lignes du fichier sous forme de liste"""

        return self.lignes[:]